@@ -4,7 +4,8 @@ from .provider import Provider
 from .resources.constants import Constants as cnts
 from colorama import Fore, Style, Back
 from IPython import get_ipython
-from IPython.display import display, HTML, Javascript, Markdown
+from IPython.display import display, HTML, Javascript, Markdown, clear_output
+import time
 import json
 from .resources.html_template import HtmlTemplate
 from .resources.static_files_manager import StaticFilesManager
@@ -32,7 +33,6 @@ import warnings
 warnings.filterwarnings('ignore')
 
 html = HtmlTemplate()
-static = StaticFilesManager()
 version = version('chartfactor')
 
 
@@ -57,6 +57,8 @@ class CFToolkit(object):
         self.__height = '400px'
         self.__iframe_height = '300px'
 
+        display(Javascript(StaticFilesManager.js('jupyter-notebook-active-cell-change')))
+
         # region Google Colab support
         try:
             from google.colab import output
@@ -141,35 +143,15 @@ class CFToolkit(object):
             get_ipython().ex("import ipykernel")
             get_ipython().ex("import re")
 
-            display(Javascript(cnts.DEFINE_EXECUTE_PYTHON_JS_FUNC))
-            display(Javascript(cnts.DEFINE_SEND_MESSAGE_JS_FUNC))
+            display(Javascript(StaticFilesManager.js('execute-python')))
+            display(Javascript(StaticFilesManager.js('send-message')))
+            clear_output()
         except Exception as e:
             print(f'{Style.BRIGHT}{Fore.RED}{e}{Style.RESET_ALL}')
 
     def __add_message_listener(self):
         try:
-            comm = cnts.EXECUTE_PYTHON_COMMAND_JS
-            df = 'event.data.df'
-            conf = 'event.data.conf'
-            get_data_sources_code = comm % (cnts.GET_DATA_SOURCES_PY_COMMAND)
-            get_data_source_meta_code = comm % ((cnts.GET_DATA_SOURCE_META_PY_COMMAND % (df, df)))
-            visualize_code = comm % ((cnts.VISUALIZE_PY_COMMAND % (df, conf)))
-            count_query_code = comm % ((cnts.COUNT_QUERY_PY_COMMAND % (df, conf)))
-            raw_query_code = comm % ((cnts.RAW_QUERY_PY_COMMAND % (df, conf)))
-
-            listener_func = \
-                cnts.DEFINE_EVENT_LISTENER_FUNC \
-                % \
-                (get_data_sources_code,
-                 get_data_source_meta_code,
-                 visualize_code,
-                 count_query_code,
-                 raw_query_code)
-
-            display(Javascript(
-                "try{ window.removeEventListener('message', window.cfMessageEventListener, false); } catch(e) {}"))
-            add_event_listener = "window.addEventListener('message', window.cfMessageEventListener = " + listener_func + ", false);"
-            display(Javascript(add_event_listener))
+            display(Javascript(StaticFilesManager.js('cf-message-event-listener')))
         except Exception as e:
             print(f'{Style.BRIGHT}{Fore.RED}{e}{Style.RESET_ALL}')
 
@@ -204,20 +186,22 @@ class CFToolkit(object):
         :return: IFrame
         """
         self.__add_js_functions()
+        time.sleep(0.1)
 
         self.__prompt_number = get_ipython().ev("len(In)-1")
+        # Getting active cell metadata to check if is a new cell
+        # or already contains a dashboard inside
+        active_cell_id = get_ipython().ev('cfpy_active_cell_id')
         iframe_id = f'iframe{self.__prompt_number}'
         self.__kernel_id = get_ipython().ev("re.search('kernel-(.*).json', ipykernel.connect.get_connection_file()).group(1)")
 
         if isinstance(self.__kernel_id, str) and self.__kernel_id != '':
             iframe_id = f'{iframe_id}_{self.__kernel_id}'
 
-        if app is None:
-            display(HTML(
-                f'<iframe id={iframe_id} name={iframe_id} src={url}dashboard.html#/new width=100% height=764 frameborder=0 allow="clipboard-write *;"></iframe>'))
-        else:
-            display(HTML(
-                f'<iframe id={iframe_id} name={iframe_id} src={url}dashboard.html#/appName={urllib.parse.quote(app)} width=100% height=764 frameborder=0 allow="clipboard-write *;"></iframe>'))
+        query_string = f'appName={urllib.parse.quote("New Application") if app is None else urllib.parse.quote(app)}&cellId={active_cell_id}'
+        time.sleep(0.3)
+        display(HTML(
+            f'<iframe id={iframe_id} name={iframe_id} src={url}dashboard.html#/{query_string} width=100% height=764 frameborder=0 allow="clipboard-write *;"></iframe>'))
 
         self.__add_message_listener()
 
@@ -298,18 +282,64 @@ class CFToolkit(object):
             raise Exception("The source() function must be called before the filters() one")
         return self
 
-    def filter(self, filter):
+    def filter(self, filter_obj):
         """
         Sets a single filter instance to be applied to the current chart.
-        :param filter: A Filter instance.
+        :param filter_obj: A Filter instance.
         :return: The current CFToolkit instance.
         """
         if self.__provider is not None:
-            self.__provider.filter(filter)
+            self.__provider.filter(filter_obj)
         else:
             raise Exception("The source() function must be called before the filter() one")
         return self
 
+    def clientFilters(self, *args):
+        """
+        Sets the client filters to be applied to the current chart.
+        :param args: The Filter instances to be applied to the current chart.
+        :return: The current CFToolkit instance.
+        """
+        if self.__provider is not None:
+            list_args = list(args)
+            if len(list_args) > 0 and isinstance(list_args[0], list):
+                parameters = list_args[0]
+            else:
+                parameters = list(args)
+            self.__provider.clientFilters(parameters)
+        else:
+            raise Exception("The source() function must be called before the clientFilters() one")
+        return self
+
+    def clientFilter(self, filter_obj):
+        """
+        Sets a single client filter instance to be applied to the current chart.
+        :param filter_obj: A Filter instance.
+        :return: The current CFToolkit instance.
+        """
+        if self.__provider is not None:
+            self.__provider.clientFilter(filter_obj)
+        else:
+            raise Exception("The source() function must be called before the clientFilter() one")
+        return self
+
+    def staticFilters(self, *args):
+        """
+        Sets the static filters to be applied to the current chart.
+        :param args: The Filter instances to be applied to the current chart.
+        :return: The current CFToolkit instance.
+        """
+        if self.__provider is not None:
+            list_args = list(args)
+            if len(list_args) > 0 and isinstance(list_args[0], list):
+                parameters = list_args[0]
+            else:
+                parameters = list(args)
+            self.__provider.staticFilters(parameters)
+        else:
+            raise Exception("The source() function must be called before the clientFilters() one")
+        return self
+
     def metrics(self, *args):
         """
         Sets the metrics to be applied to the current chart.
@@ -479,7 +509,7 @@ class CFToolkit(object):
                 if isinstance(self.__kernel_id, str) and self.__kernel_id != '':
                     iframe_id = f'{iframe_id}_{self.__kernel_id}'
 
-                style = static.css()
+                style = StaticFilesManager.css()
                 if self.__width:
                     if isinstance(self.__width, int):
                         self.__width = f'{self.__width}px'
