@@ -4,6 +4,7 @@ from .legend import Legend
 from .grid import Grid
 from .markline import MarkLine
 from .color import Color
+from random import randint
 
 eol = '\n'
 tab = '\t'
@@ -24,6 +25,7 @@ class CodeGenerator(object):
 
     def __init__(self, cf):
         self.__cf = cf
+        self.__current_filter_number = 0
 
     def __get_header_code(self):
         declaration_code = '/* Configuration code for this widget */' + eol
@@ -167,10 +169,11 @@ class CodeGenerator(object):
                             palette_call = f'{eol}{tab}.palette({json.dumps(o.get_palette)})' if o.get_palette is not None else ''
                             metric_call = f'{eol}{tab}.metric(metricColor)' if o.get_metric is not None else ''
                             range_call = f'{eol}{tab}.range({json.dumps(o.get_range)})' if o.get_range is not None else ''
+                            match_call = f'{eol}{tab}.match({json.dumps(o.get_match)})' if o.get_match is not None else ''
                             theme_call = f'{eol}{tab}.theme({json.dumps(o.get_theme)})' if o.get_theme is not None else ''
                             auto_range_call = f'{eol}{tab}.autoRange({json.dumps(o.get_auto_range_options)})' if o.get_auto_range_options is not None else ''
 
-                            declaration_extra += f'{COMMENT_COLOR}{eol}let color = cf.Color(){palette_call}{metric_call}{range_call}{theme_call}{auto_range_call};{eol}'
+                            declaration_extra += f'{COMMENT_COLOR}{eol}let color = cf.Color(){palette_call}{metric_call}{range_call}{theme_call}{auto_range_call}{match_call};{eol}'
                             set_statement += f'{eol}{tab}.set("color", color)'
                         else:
                             raise Exception('The value for color must be an instance of Color object')
@@ -182,8 +185,8 @@ class CodeGenerator(object):
                         # Processing the Color object specified in the 'widgetProps' -> 'props' -> 'color' property
                         default = lambda o: o.toJs() if isinstance(o, Color) else o
                         o = json.dumps(o, default=default)
-                        o = re.sub(r"\"cf.", 'cf.', o)
-                        o = re.sub(r"\)\"", ')', o)
+                        o = re.sub(r"\"__cf.", 'cf.', o)
+                        o = re.sub(r"\)__\"", ')', o)
                         o = re.sub(r"\\", '', o)
                         set_statement += f'{eol}{tab}.set("columnStats", {o})'
                     elif k == 'dataZoom':
@@ -213,16 +216,16 @@ class CodeGenerator(object):
         filters_names = ''
 
         if len(filters) > 0:
-            filters_code += COMMENT_FILTERS + eol
-            fn = 0
+            filters_code += COMMENT_FILTERS + eol            
             for f in filters:
+                fn = self.__current_filter_number
                 if f.get_relative:
                     filters_code += f'let filter{fn} = cf.Filter("{f.get_path}"){eol}{tab}.value("{f.get_value}").isRelative();'
                 else:
                     is_text_filter = f'{eol}{tab}.isTextFilter()' if f.is_text_filter else ''
                     filters_code += f'let filter{fn} = cf.Filter("{f.get_path}"){eol}{tab}.label("{f.get_label}"){eol}{tab}.operation("{f.get_operation}"){eol}{tab}.value({json.dumps(f.get_value)}){is_text_filter};' + eol
                 filters_names += f'filter{fn},'
-                fn += 1
+                self.__current_filter_number += 1
         return [filters_code, filters_names]
 
     def __generate_metrics_code(self, metrics):
@@ -430,6 +433,8 @@ class CodeGenerator(object):
         try:
             prov = self.__cf.get_provider
             filters = prov.get_filters
+            client_filters = prov.get_client_filters
+            static_filters = prov.get_static_filters
             metrics = prov.get_metrics
             comparative_metrics = prov.get_comparative_metrics
             attrs = prov.get_groups
@@ -452,6 +457,14 @@ class CodeGenerator(object):
             filters_code = self.__generate_filters_code(filters)
             code += filters_code[0]
             filters_call = f'{eol}{tab}.filters({filters_code[1][:-1]})' if filters_code[1] != '' else ''
+            # Client filters
+            client_filters_code = self.__generate_filters_code(client_filters)
+            code += client_filters_code[0]
+            client_filters_call = f'{eol}{tab}.clientFilters({client_filters_code[1][:-1]})' if client_filters_code[1] != '' else ''
+            # Static filters
+            static_filters_code = self.__generate_filters_code(static_filters)
+            code += static_filters_code[0]
+            static_filters_call = f'{eol}{tab}.staticFilters({static_filters_code[1][:-1]})' if static_filters_code[1] != '' else ''
             # Metrics and Comparative Metrics
             metrics_code = self.__generate_metrics_code(metrics)
             code += metrics_code[0]
@@ -489,7 +502,7 @@ class CodeGenerator(object):
 
             code += declaration_extra
             code += COMMENT_DATA + eol
-            code += f'let myData = source{filters_call}{metrics_call}{attributes_call}{rows_code}{columns_code}{fields_code}{exclude_code};' + eol + eol
+            code += f'let myData = source{filters_call}{client_filters_call}{static_filters_call}{metrics_call}{attributes_call}{rows_code}{columns_code}{fields_code}{exclude_code};' + eol + eol
             code += f'myData.graph("{chart}")'
             code += limit_code + location_code + precision_code + time_field_code
             code += set_statement + eol
