@@ -64,6 +64,11 @@ def transform_columns(columns):
             new_columns.append('::'.join(filter(None, c)))
         else:
             new_columns.append(c)
+
+    for i in range(len(new_columns)):
+        c = re.sub(r'\s*(::size)$', '::count', new_columns[i])
+        c = re.sub(r'\s*(::nunique)$', '::unique', c)
+        new_columns[i] = c
     return new_columns
 
 
@@ -91,8 +96,19 @@ def convert_to_datetime_efficiently(df, datetime_col):
     df[datetime_col] = df[datetime_col].map(m)
 
 
+def replace_agg(string):
+    replaced = re.sub(r'\s*(_size)$', '_count', string)
+    replaced = re.sub(r'\s*(_nunique)$', '_unique', replaced)
+    return replaced
+
+
 def sanitize(string):
-    return re.sub(r'[\.\,\;\:\'\"\s\%\(\)\[\]\{\}\|\/\*\+\-\=\$\@\!\#\^]*', '', string)
+    sanitized = re.sub(r'[\.\,\;\:\'\"\s\%\(\)\[\]\{\}\|\/\*\+\-\=\$\@\!\#\^\&]*', '', string)
+    sanitized = ''.join(c for c in sanitized if c.isalnum() or c == '_')
+    sanitized = replace_agg(sanitized)
+    if sanitized[0].isdigit():
+        sanitized = f'cf_{sanitized}'
+    return sanitized
 
 
 def add_one_at_last_digit(input_string):
@@ -101,6 +117,13 @@ def add_one_at_last_digit(input_string):
     return dec.next_plus()
 
 
+def translate_nullable_value(value):
+    if str(value) in ['None', 'nan']:
+        return None
+    else:
+        return value
+
+
 class Provider(object):
     """
     Sets the query's parameters defined in the json config.
@@ -111,6 +134,8 @@ class Provider(object):
         self.__data = dataframe
         self.__config = json_config
         self.__filters = []
+        self.__client_filters = []
+        self.__static_filters = []
         self.__groups = []
         self.__metrics = []
         self.__original_metrics = []
@@ -169,6 +194,8 @@ class Provider(object):
 
     def __clear_attrs__(self):
         self.__filters = []
+        self.__client_filters = []
+        self.__static_filters = []
         self.__groups = []
         self.__fields = []
         self.__exclude = []
@@ -196,6 +223,14 @@ class Provider(object):
     def get_filters(self):
         return self.__filters
 
+    @property
+    def get_client_filters(self):
+        return self.__client_filters
+
+    @property
+    def get_static_filters(self):
+        return self.__static_filters
+
     @property
     def get_rows(self):
         return self.__rows
@@ -265,6 +300,7 @@ class Provider(object):
     def __process_json_config__(self):
         try:
             if isinstance(self.__data, pd.core.frame.DataFrame):
+                self.__metadata = json.loads(self.get_data_source('name'))
                 if self.__config and self.__validate_json_config__():
                     json_string = json.dumps(self.__config)
                     json_data = json.loads(json_string)
@@ -453,6 +489,14 @@ class Provider(object):
                     if self.__time_zone.lower() == 'local':
                         self.__time_zone = tzlocal.get_localzone().zone
                     self.__has_comparative = json_data.get('hasComparative')
+                    self.__slicerTextFilter = cmms.__deep_get__(json_data, 'config.slicerTextFilter')
+                    if self.__slicerTextFilter:
+                        slicer_group = self.__groups[0].get_name
+                        slicer_text_filter = Filter(slicer_group).operation(cnts.TS).value(self.__slicerTextFilter)
+                        self.__filters.append(slicer_text_filter)
+                        for f in self.__filters:
+                            if f.get_path == slicer_group and f.get_operation == cnts.IN:
+                                f.operation(cnts.NOT_IN)
                     self.__error = False
                 else:
                     self.__error = True
@@ -477,7 +521,10 @@ class Provider(object):
                 label = g.get('label') if g.get('label') is not None else g.get('name')
 
                 metric_name = cmms.__deep_get__(g, 'sort.name')
-                metric_func = str.lower(cmms.__deep_get__(g, 'sort.func') or '')
+                metric_func = cmms.__deep_get__(g, 'sort.metricFunc')
+                if metric_func is None or metric_func == '':
+                    metric_func = cmms.__deep_get__(g, 'sort.func')
+                metric_func = str.lower(metric_func or 'sum')
 
                 if metric_name == g.get('name'):
                     metric_func = cnts.DISTINCT
@@ -584,31 +631,27 @@ class Provider(object):
 
     # region Helper functions
     def is_field_type(self, name, type):
-        metadata = json.loads(self.get_data_source('name'))
-        object_fields = metadata.get('objectFields')
+        object_fields = self.__metadata.get('objectFields')
         if object_fields:
             return any(f for f in object_fields if f.get('name') == name and f.get('type') == type)
         return False
 
     def get_original_type(self, name):
-        metadata = json.loads(self.get_data_source('name'))
-        object_fields = metadata.get('objectFields')
+        object_fields = self.__metadata.get('objectFields')
         if object_fields:
             field = list(filter(lambda f: f.get('name') == name, object_fields))
             return field[0]['originalType']
         return None
 
     def get_time_fields(self):
-        metadata = json.loads(self.get_data_source('name'))
-        object_fields = metadata.get('objectFields')
+        object_fields = self.__metadata.get('objectFields')
         if object_fields:
             fields = list(filter(lambda f: f.get('type') == 'TIME', object_fields))
             return fields
         return None
 
     def get_chartfactor_type(self, name):
-        metadata = json.loads(self.get_data_source('name'))
-        object_fields = metadata.get('objectFields')
+        object_fields = self.__metadata.get('objectFields')
         if object_fields:
             field = list(filter(lambda f: f.get('name') == name, object_fields))
             return field[0]['type']
@@ -636,7 +679,7 @@ class Provider(object):
                         converted_values = []
                         for v in f.get_value:
                             try:
-                                if cf_type == 'ATTRIBUTE':
+                                if cf_type == 'ATTRIBUTE' and v.lower() in ('true', 'false'):
                                     v = bool(util.strtobool(v))
                                 elif cf_type == 'INTEGER':
                                     v = int(float(v))
@@ -846,12 +889,12 @@ class Provider(object):
                     if m.name in metrics:
                         metrics[m.name].append(m.func)
                     else:
-                        metrics[m.name] = [unique] if m.func == cnts.UNIQUE else [count] if m.func == cnts.COUNT else [
+                        metrics[m.name] = ['nunique'] if m.func == cnts.UNIQUE else ['size'] if m.func == cnts.COUNT else [
                             m.func]
 
                 if 'total_cf_count' not in metrics:
                     self.__metrics.append(Metric('total_cf_count', 'count'))
-                    metrics['total_cf_count'] = [count]
+                    metrics['total_cf_count'] = ['size']
 
                 # Taking only the columns that will be used
                 metrics_df = self.__data.loc[:, self.__data.columns.isin(columns)].reset_index(drop=True)
@@ -874,7 +917,11 @@ class Provider(object):
                             if not pd.isna(row[m.name]):
                                 if m.name not in current_metrics:
                                     current_metrics[m.name] = {}
-                                current_metrics[m.name][index.replace('mean', 'avg')] = round(float(row[m.name]), 2)
+                                metric_func = index.replace('mean', 'avg').replace('nunique', 'unique')
+                                metric_value = translate_nullable_value(row[m.name])
+                                if metric_value is not None:
+                                    metric_value = round(float(row[m.name]), 2)
+                                current_metrics[m.name][metric_func] = metric_value
 
                 response.get('data').append({
                     'group': [],
@@ -910,15 +957,15 @@ class Provider(object):
                 if m.name in metrics:
                     metrics[m.name].append(m.func)
                 else:
-                    metrics[m.name] = [unique] if m.func == cnts.UNIQUE else [count] if m.func == cnts.COUNT else [
+                    metrics[m.name] = ['nunique'] if m.func == cnts.UNIQUE else ['size'] if m.func == cnts.COUNT else [
                         m.func]
 
             # Adding total count
             if g.get_name in metrics:
-                if count not in metrics[g.get_name]:
-                    metrics[g.get_name].append(count)
+                if 'size' not in metrics[g.get_name]:
+                    metrics[g.get_name].append('size')
             else:
-                metrics[g.get_name] = [count]
+                metrics[g.get_name] = ['size']
 
             # Taking only the columns that will be used
             group_df = self.__data.loc[:, self.__data.columns.isin(columns)].reset_index(drop=True)
@@ -932,13 +979,16 @@ class Provider(object):
                 group_df[g.get_name] = group_df[g.get_name].dt.tz_localize(None)  # Removing time offset
                 group_df[g.get_name] = group_df[g.get_name].dt.to_period(granularity_g).dt.to_timestamp()
 
-            group = group_df.groupby(g.get_name, dropna=False).agg(metrics).copy()
-            group = group.sort_values([(sort_name, sort_func)], ascending=[asc]).iloc[:limit].reset_index()
+            group = group_df.groupby(g.get_name, dropna=False).agg(metrics).copy().reset_index()
             group.columns = [sanitize('_'.join(filter(None, x))) for x in group.columns]  # Renaming columns
 
             cf_type = self.get_chartfactor_type(g.get_name)
-            if cf_type != 'ATTRIBUTE' and g.get_name == sort_name:
-                group = group.sort_values([sanitize(g.get_name)], ascending=[asc]).reset_index(drop=True)
+            if (cf_type != 'ATTRIBUTE' and g.get_name == sort_name) or \
+                    (cf_type == 'ATTRIBUTE' and sort_func == cnts.UNIQUE):
+                group = group.sort_values([sanitize(g.get_name)], ascending=[asc]).iloc[:limit].reset_index(drop=True)
+            else:
+                group = group.sort_values([f'{sanitize(sort_name)}_{sort_func}'], ascending=[asc]).iloc[:limit].reset_index(drop=True)
+
             if self.__visualization == 'Disk':
                 group = group.sort_values([sanitize(sort_name)], ascending=[True]).reset_index(drop=True)
 
@@ -954,12 +1004,19 @@ class Provider(object):
                     if m.func != cnts.COUNT:
                         if m.name not in current_metrics:
                             current_metrics[m.name] = {}
-                        current_metrics[m.name][m.func.replace('mean', 'avg')] = round(float(row[f'{sanitize(m.name)}_{m.func}']), 2)
 
-                if str(row[sanitize(g.get_name)]) in ['None', 'nan']:
-                    row_value = None
-                else:
-                    row_value = row[sanitize(g.get_name)] if self.__visualization == 'Disk' else str(row[sanitize(g.get_name)])
+                        metric_value = translate_nullable_value(row[f'{sanitize(m.name)}_{m.func}'])
+                        if metric_value is not None:
+                            metric_value = round(float(row[f'{sanitize(m.name)}_{m.func}']), 2)
+                        current_metrics[m.name][m.func.replace('mean', 'avg')] = metric_value
+
+                row_value = translate_nullable_value(row[sanitize(g.get_name)])
+                if row_value is not None:
+                    row_value = row[sanitize(g.get_name)]
+                    # Converting to string the fields to avoid json serialize exception
+                    cf_type = self.get_chartfactor_type(g.get_name)
+                    if cf_type not in ['INTEGER', 'NUMBER']:
+                        row_value = str(row_value)
 
                 response.get('data').append({
                     'group': [row_value],
@@ -1002,15 +1059,15 @@ class Provider(object):
                 if m.name in metrics:
                     metrics[m.name].append(m.func)
                 else:
-                    metrics[m.name] = [unique] if m.func == cnts.UNIQUE else [count] if m.func == cnts.COUNT else [
+                    metrics[m.name] = ['nunique'] if m.func == cnts.UNIQUE else ['size'] if m.func == cnts.COUNT else [
                         m.func]
 
             # Adding total count
             if g1.get_name in metrics:
-                if count not in metrics[g1.get_name]:
-                    metrics[g1.get_name].append(count)
+                if 'size' not in metrics[g1.get_name]:
+                    metrics[g1.get_name].append('size')
             else:
-                metrics[g1.get_name] = [count]
+                metrics[g1.get_name] = ['size']
 
             groups_df = self.__data.loc[:, self.__data.columns.isin(columns)].reset_index(drop=True)
             groups_df['total_cf_count'] = 1  # Column to calculate the count in queries
@@ -1032,17 +1089,18 @@ class Provider(object):
                 groups_df[g2.get_name] = groups_df[g2.get_name].dt.to_period(granularity_g2).dt.to_timestamp()
 
             group_by_array = [g1.get_name, g2.get_name] if g1.get_name != g2.get_name else [g1.get_name]
-            sub_query1 = groups_df.groupby(group_by_array, dropna=False).agg(metrics).copy()
-            sub_query1 = sub_query1.sort_values([(sort_name_g2, sort_func_g2)], ascending=[asc_g2]).reset_index()
+            sub_query1 = groups_df.groupby(group_by_array, dropna=False).agg(metrics).copy().reset_index()
             sub_query1.columns = [sanitize('_'.join(filter(None, x))) for x in sub_query1.columns]
+            sub_query1 = sub_query1.sort_values([f'{sanitize(sort_name_g2)}_{sort_func_g2}'], ascending=[asc_g2]).reset_index()
 
             agg_sub_query2 = {sort_name_g1: [sort_func_g1]}
-            sub_query2 = groups_df.groupby([g1.get_name], dropna=False).agg(agg_sub_query2).copy()
-            sub_query2 = sub_query2.sort_values([(sort_name_g1, sort_func_g1)], ascending=[asc_g1]).iloc[
-                         :limit_g1].reset_index()
-            if sort_name_g1 == g1.get_name:
-                sub_query2 = sub_query2.sort_values([(sort_name_g1, '')], ascending=[asc_g1]).reset_index(drop=True)
+            sub_query2 = groups_df.groupby([g1.get_name], dropna=False).agg(agg_sub_query2).copy().reset_index()
             sub_query2.columns = [sanitize('_'.join(filter(None, x))) for x in sub_query2.columns]
+            sort_by = f'{sanitize(sort_name_g1)}_{sort_func_g1}'
+            if sort_name_g1 == g1.get_name:
+                sort_by = sanitize(sort_name_g1)
+            sub_query2 = sub_query2.sort_values([sort_by], ascending=[asc_g1]).iloc[
+                         :limit_g1].reset_index(drop=True)
 
             metrics_select = ""
             for m in self.__metrics:
@@ -1251,7 +1309,7 @@ class Provider(object):
 
                 # Adding the groups intervals into the group column
                 metric_df['group'] = np.floor((metric_df[m.name] - offset) / interval) * interval + offset
-                group = metric_df.groupby('group', dropna=False).agg({m.name: [count]}).reset_index()
+                group = metric_df.groupby('group', dropna=False).agg({m.name: ['size']}).reset_index()
                 group.columns = [sanitize('_'.join(filter(None, x))) for x in group.columns]
                 if self.__limit > 0:
                     limited_group = group.iloc[:self.__limit]
@@ -1489,15 +1547,15 @@ class Provider(object):
                     if m.name in metrics:
                         metrics[m.name].append(m.func)
                     else:
-                        metrics[m.name] = [unique] if m.func == cnts.UNIQUE else [count] if m.func == cnts.COUNT else [
+                        metrics[m.name] = ['nunique'] if m.func == cnts.UNIQUE else ['size'] if m.func == cnts.COUNT else [
                             m.func]
 
                 # Adding total count
                 if r[0] in metrics:
-                    if count not in metrics[r[0]]:
-                        metrics[r[0]].append(count)
+                    if 'size' not in metrics[r[0]]:
+                        metrics[r[0]].append('size')
                 else:
-                    metrics[r[0]] = [count]
+                    metrics[r[0]] = ['size']
 
                 # Taking only the columns that will be used
                 group_df = self.__data.loc[:, self.__data.columns.isin(columns)].reset_index(drop=True)
@@ -1518,7 +1576,7 @@ class Provider(object):
                 else:
                     group = group.iloc[:100]
 
-                group.columns = ['_'.join(filter(None, x)) for x in group.columns]
+                group.columns = [sanitize('_'.join(filter(None, x))) for x in group.columns]
 
                 response = {
                     'data': [],
@@ -1574,7 +1632,7 @@ class Provider(object):
                     columns = [self.__location]
                     for m in self.__original_metrics:
                         columns.append(m.name)
-                        agg[m.name] = [unique] if m.func == cnts.UNIQUE else [count] if m.func == cnts.COUNT else [
+                        agg[m.name] = ['nunique'] if m.func == cnts.UNIQUE else ['size'] if m.func == cnts.COUNT else [
                             m.func]
 
                     # Taking only the columns that will be used
@@ -1587,11 +1645,11 @@ class Provider(object):
                     geo_df['$geohash'] = geo_df[self.__location].map(m)
 
                     # Adding total count
-                    agg['$geohash'] = [count]
+                    agg['$geohash'] = ['size']
 
                     group = geo_df.groupby('$geohash', dropna=False).agg(agg)
-                    group = group.sort_values([('$geohash', 'count')], ascending=[False])
                     group.columns = transform_columns(group.columns)
+                    group = group.sort_values(['$geohash::count'], ascending=[False])
 
                     response = {
                         'data': [],
@@ -1633,10 +1691,10 @@ class Provider(object):
                 if m.name in metrics:
                     metrics[m.name].append(m.func)
                 else:
-                    metrics[m.name] = [unique] if m.func == cnts.UNIQUE else [m.func]
+                    metrics[m.name] = ['nunique'] if m.func == cnts.UNIQUE else [m.func]
 
             # Adding total count
-            metrics['total_cf_count'] = [count]
+            metrics['total_cf_count'] = ['size']
 
             groups_df = self.__data.loc[:, self.__data.columns.isin(columns)].reset_index(drop=True)
             groups_df['total_cf_count'] = 1  # Column to calculate the count in queries
@@ -1802,12 +1860,41 @@ class Provider(object):
         self.__error = False
         return self
 
-    def filter(self, filter):
-        if not isinstance(filter, Filter):
+    def filter(self, filter_obj):
+        if not isinstance(filter_obj, Filter):
             print('filter function parameter must be an instance of a Filter object')
             self.__error = True
             return self
-        self.__filters.append(filter)
+        self.__filters.append(filter_obj)
+        self.__error = False
+        return self
+
+    def clientFilters(self, args):
+        for f in args:
+            if not isinstance(f, Filter):
+                print('clientFilters function parameters must be an instance of a Filter object')
+                self.__error = True
+                return self
+            self.__client_filters.append(f)
+        self.__error = False
+        return self
+
+    def clientFilter(self, filter_obj):
+        if not isinstance(filter_obj, Filter):
+            print('clientFilter function parameter must be an instance of a Filter object')
+            self.__error = True
+            return self
+        self.__client_filters.append(filter_obj)
+        self.__error = False
+        return self
+
+    def staticFilters(self, args):
+        for f in args:
+            if not isinstance(f, Filter):
+                print('staticFilters function parameters must be an instance of a Filter object')
+                self.__error = True
+                return self
+            self.__static_filters.append(f)
         self.__error = False
         return self
 
