@@ -20,12 +20,18 @@ class TestProvider(unittest.TestCase):
         path_ctt = os.path.dirname(os.path.realpath(__file__)) + '/csv/chicago_taxi_trips.csv'
         path_lfb = os.path.dirname(os.path.realpath(__file__)) + '/csv/london_fire_brigade.csv'
         path_ttn = os.path.dirname(os.path.realpath(__file__)) + '/csv/titanic.csv'
+        path_ifr = os.path.dirname(os.path.realpath(__file__)) + '/csv/funding_rounds.csv'
+        path_hlc = os.path.dirname(os.path.realpath(__file__)) + '/csv/healthy_lifestyle_city_2021.csv'
+        path_dbr = os.path.dirname(os.path.realpath(__file__)) + '/csv/Data_Breaches_EN_V2_2004_2017_20180220.csv'
         ts = pd.read_csv(path, low_memory=False, parse_dates=['saletime'], infer_datetime_format=True)
         cls.df = ts
         cls.df_ctt = pd.read_csv(path_ctt, low_memory=False)
         cls.df_lfb = pd.read_csv(path_lfb, low_memory=False, parse_dates=['date_of_call', 'timestamp_of_call'], infer_datetime_format=True)
         cls.df_lfb['date_of_call'] = pd.to_datetime(cls.df_lfb['date_of_call']).dt.date
         cls.df_ttn = pd.read_csv(path_ttn, low_memory=False)
+        cls.df_ifr = pd.read_csv(path_ifr, low_memory=False)
+        cls.df_hlc = pd.read_csv(path_hlc, low_memory=False)
+        cls.df_dbr = pd.read_csv(path_dbr, low_memory=False)
         cls.prov = Provider(ts, prov_data.JSON_CONFIG)
 
     def setUp(self):
@@ -2003,6 +2009,353 @@ class TestProvider(unittest.TestCase):
 
         result = json.loads(Provider(self.df_lfb, config).run_raw_query()).get('data')
         self.assertDictEqual(result, prov_data.LFB_RUN_RAW_QUERY_WITH_DATE_FILTER)
+
+    def test_slicer_with_text_filter_only(self):
+        config = {
+            "uid": "0575a12e-3394-4e7d-bd16-0497d45da57f",
+            "providerName": "Pandas DataFrames",
+            "aqlId": "__DEFAULT__",
+            "timezone": {},
+            "config": {
+                "filters": [],
+                "staticFilters": [],
+                "clientFilters": [],
+                "textFilter": "",
+                "staticTextFilter": "",
+                "groups": [
+                    {
+                        "name": "name",
+                        "label": "name",
+                        "limit": 10000,
+                        "sort": {
+                            "dir": "asc",
+                            "name": "name"
+                        },
+                        "is": "Attribute",
+                        "type": "ATTRIBUTE",
+                        "originName": "name",
+                        "originalType": "string"
+                    }
+                ],
+                "metrics": [
+                    {
+                        "name": "investor_count",
+                        "func": "sum",
+                        "customLabel": False,
+                        "label": "investor_count",
+                        "hideFunction": False,
+                        "is": "Metric",
+                        "type": "NUMBER",
+                        "originName": "investor_count",
+                        "originalType": "floating"
+                    }
+                ],
+                "comparative": [],
+                "slicerTextFilter": "Stev"
+            },
+            "volumeMetric": {
+                "label": "Transactions",
+                "func": "",
+                "type": "INTEGER",
+                "name": "count"
+            },
+            "hasDerivedFields": False,
+            "aggregate": False,
+            "limit": None,
+            "offset": 0,
+            "originalGroup": {
+                "name": "name",
+                "label": "",
+                "limit": 10000
+            },
+            "visualization": "Slicer",
+            "element": "visefbe8f8e-5eac-4d2d-9016-f1850706ec66",
+            "source": {
+                "name": "funding_rounds"
+            },
+            "color": {
+                "_metric": None
+            }
+        }
+
+        result = json.loads(Provider(self.df_ifr, config).visualize()).get('data')
+        self.assertDictEqual(result, prov_data.IFR_SLICER_WITH_TEXT_FILTER)
+
+    def test_slicer_with_text_filter_and_in_filters(self):
+        config = {
+            "uid": "0575a12e-3394-4e7d-bd16-0497d45da57f",
+            "providerName": "Pandas DataFrames",
+            "aqlId": "__DEFAULT__",
+            "timezone": {},
+            "config": {
+                "filters": [
+                    {
+                        "path": "name",
+                        "label": "name",
+                        "enabled": True,
+                        "operation": "IN",
+                        "value": [
+                            "Angel Round - !Creatice"
+                        ],
+                        "relative": False,
+                        "presetValue": False,
+                        "isTextFilter": False,
+                        "id": "26ff2525-b252-45d9-b878-7d25242cadf5",
+                        "sender": {
+                            "type": "Slicer",
+                            "id": "visefbe8f8e-5eac-4d2d-9016-f1850706ec66"
+                        },
+                        "origin": {
+                            "name": "name",
+                            "label": "name",
+                            "limit": 10000,
+                            "sort": {
+                                "dir": "asc",
+                                "name": "name"
+                            },
+                            "is": "Attribute",
+                            "type": "ATTRIBUTE",
+                            "originName": "name",
+                            "originalType": "string"
+                        }
+                    }
+                ],
+                "staticFilters": [],
+                "clientFilters": [],
+                "textFilter": "",
+                "staticTextFilter": "",
+                "groups": [
+                    {
+                        "name": "name",
+                        "label": "name",
+                        "limit": 10000,
+                        "sort": {
+                            "dir": "asc",
+                            "name": "name"
+                        },
+                        "is": "Attribute",
+                        "type": "ATTRIBUTE",
+                        "originName": "name",
+                        "originalType": "string"
+                    }
+                ],
+                "metrics": [
+                    {
+                        "name": "investor_count",
+                        "func": "sum",
+                        "customLabel": False,
+                        "label": "investor_count",
+                        "hideFunction": False,
+                        "is": "Metric",
+                        "type": "NUMBER",
+                        "originName": "investor_count",
+                        "originalType": "floating"
+                    }
+                ],
+                "comparative": [],
+                "slicerTextFilter": "Stev"
+            },
+            "volumeMetric": {
+                "label": "Transactions",
+                "func": "",
+                "type": "INTEGER",
+                "name": "count"
+            },
+            "hasDerivedFields": False,
+            "aggregate": False,
+            "limit": None,
+            "offset": 0,
+            "originalGroup": {
+                "name": "name",
+                "label": "",
+                "limit": 10000
+            },
+            "visualization": "Slicer",
+            "element": "visefbe8f8e-5eac-4d2d-9016-f1850706ec66",
+            "source": {
+                "name": "funding_rounds"
+            },
+            "color": {
+                "_metric": None
+            }
+        }
+
+        result = json.loads(Provider(self.df_ifr, config).visualize()).get('data')
+        self.assertDictEqual(result, prov_data.IFR_SLICER_WITH_TEXT_FILTER)
+
+    def test_rdt_kpi_column_stats(self):
+        config = {
+            "uid": "0c65698e-8ba1-4bb6-a402-44b728350084",
+            "providerName": "Pandas DataFrames",
+            "aqlId": "__DEFAULT__",
+            "timezone": {},
+            "config": {
+                "filters": [],
+                "staticFilters": [],
+                "clientFilters": [],
+                "textFilter": "",
+                "staticTextFilter": "",
+                "groups": [],
+                "metrics": [
+                    {
+                        "name": "count",
+                        "func": "",
+                        "customLabel": False,
+                        "label": "Transactions",
+                        "hideFunction": False,
+                        "is": "Metric",
+                        "type": "INTEGER"
+                    }
+                ],
+                "comparative": []
+            },
+            "volumeMetric": {
+                "label": "Transactions",
+                "func": "",
+                "type": "INTEGER",
+                "name": "count"
+            },
+            "hasDerivedFields": False,
+            "aggregate": False,
+            "limit": None,
+            "offset": 0,
+            "element": "6b1bd997-3008-4943-a802-0ab8b6222875",
+            "source": {
+                "name": "healthy_lifestyle_cities"
+            },
+            "color": {
+                "_metric": None
+            }
+        }
+
+        result = json.loads(Provider(self.df_hlc, config).visualize()).get('data')
+        self.assertDictEqual(result, prov_data.HLC_RDT_KPI_COLUMN_STAT)
+
+    def test_rdt_kpi_unique_column_stats(self):
+        config = {
+            "uid": "3e03a8b1-89d3-493e-8b0c-fafb77b139b0",
+            "providerName": "Pandas DataFrames",
+            "_parentId": "visualization",
+            "_isChild": True,
+            "aqlId": "__DEFAULT__",
+            "timezone": {},
+            "config": {
+                "filters": [],
+                "staticFilters": [],
+                "clientFilters": [],
+                "textFilter": "",
+                "staticTextFilter": "",
+                "groups": [],
+                "metrics": [
+                    {
+                        "name": "City",
+                        "func": "unique",
+                        "customLabel": False,
+                        "label": "City",
+                        "hideFunction": False,
+                        "is": "Metric",
+                        "type": "ATTRIBUTE",
+                        "originName": "City",
+                        "originalType": "string"
+                    }
+                ],
+                "comparative": []
+            },
+            "volumeMetric": {
+                "label": "Transactions",
+                "func": "",
+                "type": "INTEGER",
+                "name": "count"
+            },
+            "hasDerivedFields": False,
+            "aggregate": False,
+            "limit": None,
+            "offset": 0,
+            "element": "6d989af2-fab8-410b-a522-46b5ad822b1b",
+            "source": {
+                "name": "healthy_lifestyle_cities"
+            },
+            "color": {
+                "_metric": None
+            }
+        }
+
+        result = json.loads(Provider(self.df_hlc, config).visualize()).get('data')
+        self.assertDictEqual(result, prov_data.HLC_RDT_KPI_COLUMN_STAT_2)
+
+    def test_data_breaches_single_group_with_1st_source(self):
+        config = {
+            "uid": "ce2f21c3-5199-4e61-859a-155bdfd914d8",
+            "providerName": "Pandas DataFrames",
+            "aqlId": "__DEFAULT__",
+            "timezone": {},
+            "config": {
+                "filters": [],
+                "staticFilters": [],
+                "clientFilters": [],
+                "textFilter": "",
+                "staticTextFilter": "",
+                "groups": [
+                    {
+                        "name": "1st source",
+                        "label": "1st source",
+                        "sort": {
+                            "name": "Records Lost",
+                            "func": "sum",
+                            "customLabel": False,
+                            "label": "Records Lost",
+                            "hideFunction": False,
+                            "metricFunc": "sum",
+                            "dir": "desc",
+                            "is": "Metric",
+                            "type": "NUMBER",
+                            "originName": "Records Lost",
+                            "originalType": "floating"
+                        },
+                        "limit": 100,
+                        "is": "Attribute",
+                        "type": "ATTRIBUTE",
+                        "originName": "1st source",
+                        "originalType": "string"
+                    }
+                ],
+                "metrics": [
+                    {
+                        "name": "Records Lost",
+                        "func": "sum",
+                        "customLabel": False,
+                        "label": "Records Lost",
+                        "hideFunction": False,
+                        "is": "Metric",
+                        "type": "NUMBER",
+                        "originName": "Records Lost",
+                        "originalType": "floating"
+                    }
+                ],
+                "comparative": []
+            },
+            "volumeMetric": {
+                "label": "Transactions",
+                "func": "",
+                "type": "INTEGER",
+                "name": "count"
+            },
+            "hasDerivedFields": False,
+            "aggregate": False,
+            "limit": None,
+            "offset": 0,
+            "visualization": "Bars",
+            "element": "visualization",
+            "source": {
+                "name": "dataBreaches"
+            },
+            "color": {
+                "_metric": None
+            }
+        }
+
+        result = json.loads(Provider(self.df_dbr, config).visualize()).get('data')
+        self.assertDictEqual(result, prov_data.DBR_BREACHES_SINGLE_GROUP_WITH_1ST_SOURCE)
     # endregion
 
     # region NEGATIVE TESTS
